#!/usr/bin/python
# -*- coding: cp1251 -*-
# 2007. Cyberlight Systems
# N9 VM Management Library for QEMU and Portable VirtualBox

# v.0.4.4 beta 2
import os;
import os.path;
import md5;
import time;
import string;
import tarfile;
import locale;
import sys;
import shutil;
import signal;

#locale.setlocale(locale.LC_CTYPE, 'ru');

SYM_ARCH_X86             = 'x86';
SYM_ARCH_X86_64          = 'x86_64';
SYM_ARCH_SPARC           = 'sparc';
SYM_ARCH_PPC             = 'ppc';
SYM_ARCH_MIPS            = 'mips';
SYM_ARCH_MIPSEL          = 'mipsel';
SYM_ARCH_ARM             = 'arm';

SYM_BOOL_YES             = 'yes';
SYM_BOOL_NO              = 'no';
SYM_PARAM_AUTO           = 'auto';
SYM_PARAM_DISABLED       = 'disabled';

SYM_NIC_NE2K             = 'ne2k_isa';
SYM_NIC_NE2KPCI          = 'ne2k_pci';
SYM_NIC_RTL8139          = 'rtl8139';
SYM_NIC_SMC              = 'smc91c111';
SYM_NIC_LANCE            = 'lance';

SYM_DISPLAY_NUL          = 'NULL';
SYM_DISPLAY_VNC          = 'VNC';
SYM_DISPLAY_SDL          = 'SDL';
SYM_DISPLAY_FS           = 'FULLSCREEN';

SYM_BOOT_HDD             = 'c';
SYM_BOOT_FDD             = 'a';
SYM_BOOT_CDROM           = 'd';

SYM_MACHINE_NAME         = 'machine.name';
SYM_MACHINE_PACKAGE      = 'machine.package';
SYM_MACHINE_PLATFORM     = 'machine.platform';
SYM_MACHINE_BOOTDRIVE    = 'machine.bootdrive';
SYM_MACHINE_ACCELERATION = 'machine.acceleration';
SYM_MACHINE_VNCID        = 'machine.vncid';
SYM_MACHINE_NIC          = 'machine.nic';
SYM_MACHINE_MAC          = 'machine.mac';
SYM_MACHINE_IFTAP        = 'machine.tap';
SYM_MACHINE_MONITOR      = 'machine.monitor';
SYM_MACHINE_REDIRCFG     = 'machine.redirects.config';
SYM_MACHINE_VLAN         = 'machine.vlan.id';
SYM_MACHINE_DISPLAY      = 'machine.display';
SYM_MACHINE_STATE        = 'machine.state';
SYM_MACHINE_RAM          = 'machine.ram';
SYM_MACHINE_LINUXKERNEL  = 'machine.linux.kernel';
SYM_MACHINE_LINUXINITRD  = 'machine.linux.initrd';
SYM_MACHINE_LINUXCOMMAND = 'machine.linux.command';
SYM_MACHINE_DRIVE_HDB    = 'machine.drives.hdb';
SYM_MACHINE_DRIVE_HDC    = 'machine.drives.hdc';
SYM_MACHINE_DRIVE_HDD    = 'machine.drives.hdd';
SYM_MACHINE_DRIVE_CDROM  = 'machine.drives.cdrom';
SYM_MACHINE_BOOTFROM     = 'machine.bootfrom';

#portable virtualbox support
SYM_MACHINE_ENGINE       = 'machine.engine';
SYM_MACHINE_PVB_CONFIG   = 'machine.virtualbox.config';

SYM_GLOBAL_SMBROOT       = 'global.smbroot';
SYM_GLOBAL_VNC_CAPACITY  = 'global.vnc.capacity';
SYM_GLOBAL_VNC_BEGIN     = 'global.vnc.displays.begin';
SYM_GLOBAL_TAP           = 'global.tap.ifname';
SYM_GLOBAL_PKG_ROOT      = 'global.roots.packages';
SYM_GLOBAL_MACHINES_ROOT = 'global.roots.machines';
SYM_GLOBAL_TEMP          = 'global.roots.temp';
SYM_GLOBAL_HTTPD_ROOT    = 'global.roots.httpd';
SYM_GLOBAL_BACKUPS_ROOT  = 'global.roots.backups';
SYM_GLOBAL_QEMU          = 'global.runtime.qemu';
SYM_GLOBAL_BIOS          = 'global.runtime.bios';
SYM_GLOBAL_HTTPD_PORT    = 'global.httpd.port';
SYM_GLOBAL_XMLRPC_PORT   = 'global.xmlrpc.port';
SYM_GLOBAL_XMLRPC_OPENVZ = 'global.xmlrpc.openvz';
SYM_GLOBAL_XMLRPC_MSVS   = 'global.xmlrpc.msvs';
SYM_GLOBAL_XMLRPC_EC2    = 'global.xmlrpc.ec2';

#portable virtualbox support
SYM_GLOBAL_VIRTUALBOX    = 'global.runtime.virtualbox';
SYM_GLOBAL_VDI_ROOT      = 'global.runtime.vdi';
SYM_GLOBAL_VBOXMACHINES  = 'global.runtime.virtualbox.machines';

SYM_STATE_INACTIVE       = 'inactive';
SYM_STATE_ACTIVE         = 'active';

SYM_PIDFILE              = 'machine.pid';
SYM_VNCID_FILE           = 'machine.vnc';
SYM_VMM_FILE             = 'machine.vmm';
SYM_QVM_FILE             = 'machine.qvm';

if string.find(string.lower(sys.executable), 'python') == -1:
    FOLDER_CONFIG            = os.path.join(os.path.dirname(sys.executable),'config');
    FOLDER_PACKAGES          = os.path.join(os.path.dirname(sys.executable),'packages');
    FOLDER_MACHINES          = os.path.join(os.path.dirname(sys.executable),'machines');
    FOLDER_TEMP              = os.path.join(os.path.dirname(sys.executable),'temp');
    FOLDER_HTTPD_ROOT        = os.path.join(os.path.dirname(sys.executable), 'web');
    FOLDER_BACKUPS_ROOT      = os.path.join(os.path.dirname(sys.executable), 'backups');
else:
#uncomment for native execution
#comment for py2exe build EXEs correctly
    FOLDER_CONFIG            = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])),'config');
    FOLDER_PACKAGES          = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])),'packages');
    FOLDER_MACHINES          = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])),'machines');
    FOLDER_TEMP              = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])),'temp');
    FOLDER_HTTPD_ROOT        = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])), 'web');
    FOLDER_BACKUPS_ROOT      = os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])), 'backups');

FOLDER_BIOS              = '';
FOLDER_SMBROOT           = 'C:\\';
FOLDER_VIRTUALBOX_CORE   = '';
FOLDER_VIRTUALBOX_VDI    = '';
FOLDER_VIRTUALBOX_XML    = '';

FILE_VNCHTTPD_CONFIG     = 'httpdsec.cfg';
FILE_RESOURCE_ACCESS     = 'resources.cfg';
FILE_N9_CONFIG           = 'n9global.cfg';
FILE_VNCMAP              = 'vncmap.cfg'

CMD_QEMU                 = 'qemu';
CMD_VBOXMANAGE           = 'VBoxManage.exe';
CMD_VBOXSDL              = 'VBoxSDL.exe';
CMD_VBOXVRDP             = 'VBoxVRDP.exe';

GLOBAL_VNC_START         = 1;
GLOBAL_VNC_CAPACITY      = 100;
GLOBAL_VNC_MAP           = dict();
GLOBAL_TAP               = '';
GLOBAL_HTTPD_PORT        = 80;
GLOBAL_XMLRPC_PORT       = 8445;

QVM_SYMBOLS = (SYM_MACHINE_NAME, SYM_MACHINE_PACKAGE, SYM_MACHINE_PLATFORM,
               SYM_MACHINE_BOOTDRIVE, SYM_MACHINE_ACCELERATION, SYM_MACHINE_VNCID,
               SYM_MACHINE_NIC, SYM_MACHINE_MAC, SYM_MACHINE_IFTAP, SYM_MACHINE_MONITOR,
               SYM_MACHINE_REDIRCFG, SYM_MACHINE_VLAN, SYM_MACHINE_DISPLAY,
               SYM_MACHINE_STATE, SYM_MACHINE_RAM, SYM_MACHINE_LINUXKERNEL,
               SYM_MACHINE_LINUXINITRD, SYM_MACHINE_LINUXCOMMAND,
               SYM_MACHINE_DRIVE_HDB, SYM_MACHINE_DRIVE_HDC,SYM_MACHINE_DRIVE_HDD,
               SYM_MACHINE_DRIVE_CDROM, SYM_MACHINE_ENGINE, SYM_MACHINE_PVB_CONFIG,
               SYM_MACHINE_BOOTFROM);

def configure(fConfigPath):
    global FOLDER_PACKAGES, FOLDER_MACHINES, FOLDER_TEMP, FOLDER_HTTPD_ROOT
    global FOLDER_BIOS, FOLDER_SMBROOT, CMD_QEMU, GLOBAL_VNC_START
    global GLOBAL_VNC_CAPACITY, GLOBAL_VNC_MAP, GLOBAL_TAP, GLOBAL_HTTPD_PORT

    if not os.path.exists(os.path.join(FOLDER_CONFIG, FILE_N9_CONFIG)):
        return
    data = ParseQVM(fConfigPath);
    
    if data.has_key(SYM_GLOBAL_BACKUPS_ROOT):
        FOLDER_BACKUPS_ROOT = os.path.abspath(data[SYM_GLOBAL_BACKUPS_ROOT]);
    
    if data.has_key(SYM_GLOBAL_SMBROOT):
        FOLDER_SMBROOT = os.path.abspath(data[SYM_GLOBAL_SMBROOT]);

    if data.has_key(SYM_GLOBAL_PKG_ROOT):
        FOLDER_PACKAGES = os.path.abspath(data[SYM_GLOBAL_PKG_ROOT]);

    if data.has_key(SYM_GLOBAL_MACHINES_ROOT):
        FOLDER_MACHINES = os.path.abspath(data[SYM_GLOBAL_MACHINES_ROOT]);

    if data.has_key(SYM_GLOBAL_TEMP):
        FOLDER_TEMP = os.path.abspath(data[SYM_GLOBAL_TEMP]);

    if data.has_key(SYM_GLOBAL_HTTPD_ROOT):
        FOLDER_HTTPD_ROOT = os.path.abspath(data[SYM_GLOBAL_HTTPD_ROOT]);
        
    if data.has_key(SYM_GLOBAL_BIOS):
        FOLDER_BIOS = os.path.abspath(data[SYM_GLOBAL_BIOS]);

    if data.has_key(SYM_GLOBAL_TAP):
        GLOBAL_TAP = data[SYM_GLOBAL_TAP];

    if data.has_key(SYM_GLOBAL_VNC_BEGIN):
        GLOBAL_VNC_START = data[SYM_GLOBAL_VNC_BEGIN];

    if data.has_key(SYM_GLOBAL_VNC_CAPACITY):
        GLOBAL_VNC_CAPACITY = data[SYM_GLOBAL_VNC_CAPACITY];

    if data.has_key(SYM_GLOBAL_HTTPD_PORT):
        GLOBAL_HTTPD_PORT = data[SYM_GLOBAL_HTTPD_PORT];

    if data.has_key(SYM_GLOBAL_QEMU):
        CMD_QEMU = data[SYM_GLOBAL_QEMU];

    #portable virtualbox support
    if data.has_key(SYM_GLOBAL_VIRTUALBOX):
        FOLDER_VIRTUALBOX_CORE   = data[SYM_GLOBAL_VIRTUALBOX];
        
    if data.has_key(SYM_GLOBAL_VDI_ROOT):
        FOLDER_VIRTUALBOX_VDI    = data[SYM_GLOBAL_VDI_ROOT];
        
    if data.has_key(SYM_GLOBAL_VBOXMACHINES):
        FOLDER_VIRTUALBOX_XML    = data[SYM_GLOBAL_VBOXMACHINES];
        
def InitializeVNC(dVNC):
    global GLOBAL_VNC_MAP
    dVNC.clear();
    i = 0;
    if os.path.exists(os.path.join(FOLDER_CONFIG, FILE_VNCMAP)):
        f = open(os.path.join(FOLDER_CONFIG, FILE_VNCMAP), 'r');
        r = f.read();
        f.close();
        if len(r) > 0:
            GLOBAL_VNC_MAP = eval(r);
            return;
    for i in range(0, GLOBAL_VNC_CAPACITY - 1):
        dVNC[GLOBAL_VNC_START + i] = SYM_BOOL_NO;

def ParseQVM(filename):
    d = dict();
    f = open(filename, 'r');
    s = f.readlines();
    f.close();
    for l in s:
        p = string.split(l, '=', 1);
        d[p[0]] = string.replace(p[1], '\n', '');
    if d.has_key(SYM_MACHINE_PACKAGE):
        d[SYM_MACHINE_PACKAGE] = string.replace(d[SYM_MACHINE_PACKAGE], '.tgz', '');
    return d;

def get_ConsoleName():
    if os.name == 'nt':
        return 'con';
    else:
        return 'stdio';

def get_TAP():
    return GLOBAL_TAP;

def get_VNCID():
    global GLOBAL_VNC_MAP
    for i in GLOBAL_VNC_MAP.keys():
        if GLOBAL_VNC_MAP[i] == SYM_BOOL_NO:
            GLOBAL_VNC_MAP[i] = SYM_BOOL_YES;
            f = open(os.path.join(FOLDER_CONFIG, FILE_VNCMAP), 'w+');
            f.write(str(GLOBAL_VNC_MAP));
            f.flush();
            f.close();            
            return str(i);
    return '0';

def release_VNCID(iVNCID):
    global GLOBAL_VNC_MAP
    if GLOBAL_VNC_MAP.has_key(iVNCID):
        GLOBAL_VNC_MAP[iVNCID] = SYM_BOOL_NO;
        f = open(os.path.join(FOLDER_CONFIG, FILE_VNCMAP), 'w+');
        f.write(str(GLOBAL_VNC_MAP));
        f.flush();
        f.close();    
        return;
        

def get_MachineState(ID):
    if os.path.exists(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE)):
        return SYM_STATE_ACTIVE
    else:
        return SYM_STATE_INACTIVE;

def get_MachineFile(ID):
    return os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_QVM_FILE);

def get_MachinesList():
    s = dict();
    id = '';
    name = '';
    package = '';
    vncid = '0';
    for dirname,subdirs,files in os.walk(FOLDER_MACHINES):
        for dir in subdirs:
            id = dir;
            f = ParseQVM(os.path.join(FOLDER_MACHINES, id, SYM_QVM_FILE))
            f[SYM_MACHINE_STATE] = get_MachineState(id);
            if os.path.exists(os.path.join(FOLDER_MACHINES, id, SYM_VNCID_FILE)):
                r = open(os.path.join(FOLDER_MACHINES, id, SYM_VNCID_FILE), 'r');
                vncid = r.read();
                r.close()
                f[SYM_MACHINE_VNCID] = vncid 
            s[id] = f
    return s;

def get_PackagesList():
    s = list();
    for dirname, subdirs, files in os.walk(FOLDER_PACKAGES):
        for file in files:
            if string.find(file, '.tgz') >= 0:
                s = s + [string.replace(file, '.tgz', '')];
    return s;

def get_PackageFile(sPackage):
    return os.path.join(FOLDER_PACKAGES, sPackage) + '.tgz';

def get_CommandLine(filename, VNCOverride = False):
    platform = CMD_QEMU;
    tap = get_TAP();
    nic = '';
    redirs = None;
    monitor = get_ConsoleName();
    vnc = '0';
    acceleration = '-no-kqemu';
    bootdrive = '';
    display = '';
    res = '';
    mac = '';
    vlan = '';
    rs = '';
    ram = '';
    kernel = '';
    initrd = '';
    kernelcmd = '';
    hdb = '';
    hdc = '';
    hdd = '';
    cdrom = '';
    bootfrom = '';
    folder = os.path.dirname(os.path.abspath(filename));
    data = ParseQVM(filename);
    if data.has_key(SYM_MACHINE_BOOTFROM):
        bootfrom = '-boot ' + data[SYM_MACHINE_BOOTFROM];
        
    if data.has_key(SYM_MACHINE_PLATFORM):
        if data[SYM_MACHINE_PLATFORM] != SYM_ARCH_X86:
            platform = platform + '-' + data[SYM_MACHINE_PLATFORM];
            
    if data.has_key(SYM_MACHINE_BOOTDRIVE):
        bootdrive = data[SYM_MACHINE_BOOTDRIVE];
        
    if data.has_key(SYM_MACHINE_MONITOR):
        monitor = data[SYM_MACHINE_MONITOR];
        
    if data.has_key(SYM_MACHINE_DISPLAY):        
        k = data[SYM_MACHINE_DISPLAY]
        if k == SYM_DISPLAY_FS :
            display = '-full-screen';
        if k == SYM_DISPLAY_VNC:
            vnc = get_VNCID();
            display = '-vnc :' + vnc;
            
    if display == '' and VNCOverride:
            vnc = get_VNCID();
            display = '-vnc :' + vnc;
            
    if data.has_key(SYM_MACHINE_ACCELERATION):
        if data[SYM_MACHINE_ACCELERATION] == SYM_BOOL_YES:
            acceleration = '-kernel-kqemu';
                
    if data.has_key(SYM_MACHINE_NIC):
        nic = ',model=' + data[SYM_MACHINE_NIC];

    if data.has_key(SYM_MACHINE_MAC):
        mac = ',macaddr=' + data[SYM_MACHINE_MAC];
            
    if data.has_key(SYM_MACHINE_VLAN):
        vlan = ',vlan=' + data[SYM_MACHINE_VLAN];

    if data.has_key(SYM_MACHINE_RAM):
        ram = '-m ' + data[SYM_MACHINE_RAM];
        
    if data.has_key(SYM_MACHINE_LINUXKERNEL):
        kernel = '-kernel "' + os.path.join(folder, string.replace(data[SYM_MACHINE_LINUXKERNEL], '\n', '')) + '"';

    if data.has_key(SYM_MACHINE_LINUXINITRD):
        initrd = '-initrd "' + os.path.join(folder, string.replace(data[SYM_MACHINE_LINUXINITRD], '\n', '')) + '"';

    if data.has_key(SYM_MACHINE_LINUXCOMMAND):
        kernelcmd = '-append "' + data[SYM_MACHINE_LINUXCOMMAND] + '"';
    
    if data.has_key(SYM_MACHINE_DRIVE_HDB):
        hdb = '-hdb "' + os.path.join(folder, string.replace(data[SYM_MACHINE_DRIVE_HDB], '\n', '')) + '"'

    if data.has_key(SYM_MACHINE_DRIVE_HDC):
        hdc = '-hdc "' + os.path.join(folder, string.replace(data[SYM_MACHINE_DRIVE_HDC], '\n', '')) + '"'

    if data.has_key(SYM_MACHINE_DRIVE_HDD):
        hdd = '-hdd "' + os.path.join(folder, string.replace(data[SYM_MACHINE_DRIVE_HDD], '\n', '')) + '"'

    if data.has_key(SYM_MACHINE_DRIVE_CDROM):
        cdrom = '-cdrom "' + os.path.join(folder, string.replace(data[SYM_MACHINE_DRIVE_CDROM], '\n', '')) + '"'    
        
    if data.has_key(SYM_MACHINE_REDIRCFG):
        r = data[SYM_MACHINE_REDIRCFG];
        redirs = open(os.path.join(folder, r), 'r');
        #print r
        #print os.path.join(folder, r)
        for rl in redirs.readlines():
            rs = rs + '-redir ' + string.replace(rl, '\n', '') + ' ';
        redirs.close();
        if len(rs) == 0:
            rs = ' ';
        #print rs;
    if len(FOLDER_BIOS) == 0:
        res = [platform, acceleration, ' -localtime'];
    else:
        res = [platform, '-L "' + FOLDER_BIOS + '"', acceleration, '-localtime'];
    if len(display) > 0:
        res = res + [display];
        
    if monitor != '':
        #res = res + ['-monitor ' + monitor]
        pass;
        
    if ram != '':
        res = res + [ram]

    if kernel != '':
        res = res + [kernel];

    if initrd != '':
        res = res + [initrd];

    if kernelcmd != '':
        res = res + [kernelcmd];

    if hdb != '':
        res = res + [hdb];

    if hdc != '':
        res = res + [hdc];

    if hdd != '':
        res = res + [hdd];

    if cdrom != '':
        res = res + [cdrom]

    if len(tap) > 0:
        res = res + ['-net tap,ifname=' + tap];
    else:
        res = res + ['-net user']

    if nic != '' or mac != '' or vlan != '':
        rt = '-net nic'
        if vlan != '':
            rt = rt + vlan;
        if mac != '' :
            rt = rt + mac;
        if nic != '':
            rt = rt + nic;
        res = res + [rt];
            
    res = res + [rs]
    if bootdrive != '':
        res = res +  ['-hda "' + os.path.join(folder, string.replace(bootdrive, '\n', '')) + '"'];

    if bootfrom != '':
        res = res + [bootfrom]
        
    return [res, vnc, monitor];

def initialize_Config():
    d = dict();
    for k in QVM_SYMBOLS:
        d[k] = '';
    return d;

def create_Config(dParams):
    s = '';
    for i in dParams.keys():
        if len(dParams[i]) > 0:
            s = s + i + '=' + dParams[i] + '\n';
    return s;

def create_Package(dParams, path):
    folder = os.path.dirname(os.path.abspath(path));
    QVM = os.path.join(folder, SYM_QVM_FILE);
    pkg = string.replace(os.path.basename(path), '.tgz', '');
    tar = tarfile.open(os.path.abspath(path), 'w:gz');
    flag = False
    todel = []
    if dParams.has_key(SYM_MACHINE_REDIRCFG):
        if dParams[SYM_MACHINE_REDIRCFG] != '':
            srcredir = os.path.abspath(dParams[SYM_MACHINE_REDIRCFG]);
            destredir = os.path.join(folder, os.path.basename(srcredir));
            if not os.path.exists(destredir):
                shutil.copyfile(srcredir, destredir);
                dParams[SYM_MACHINE_REDIRCFG] = os.path.basename(destredir);
            tar.add(destredir, dParams[SYM_MACHINE_REDIRCFG]);
            flag = True
            
    if  dParams.has_key(SYM_MACHINE_BOOTFROM) and dParams.has_key(SYM_MACHINE_DRIVE_CDROM) and dParams.has_key(SYM_MACHINE_BOOTDRIVE):
        if dParams[SYM_MACHINE_DRIVE_CDROM] == dParams[SYM_MACHINE_BOOTDRIVE] and dParams[SYM_MACHINE_BOOTFROM] == SYM_BOOT_CDROM:
            del dParams[SYM_MACHINE_BOOTDRIVE]
                        
    for k in (SYM_MACHINE_BOOTDRIVE, SYM_MACHINE_DRIVE_HDB,
              SYM_MACHINE_DRIVE_HDC, SYM_MACHINE_DRIVE_HDD,
              SYM_MACHINE_DRIVE_CDROM, SYM_MACHINE_LINUXKERNEL,
              SYM_MACHINE_LINUXINITRD):
        if dParams.has_key(k):
            if dParams[k] != '':
                srcdrive = os.path.abspath(dParams[k]);
                destdrive = os.path.join(folder, os.path.basename(srcdrive));
                if not os.path.exists(destdrive):
                    shutil.copyfile(srcdrive, destdrive);
                todel = todel + [destdrive];
                dParams[k] = os.path.basename(destdrive);
                tar.add(destdrive, dParams[k]);

    dParams[SYM_MACHINE_PACKAGE] = pkg;
    fQVM = open(QVM, 'w+');
    for i in dParams.keys():
        if len(dParams[i]) > 0:
            s = i + '=' + dParams[i] + '\n';
            fQVM.write(s);
    fQVM.flush()
    fQVM.close();
    tar.add(QVM, SYM_QVM_FILE);
    tar.close();
    for i in todel:
        os.remove(i);
    os.remove(QVM);
    if flag:
        os.remove(destredir);
    return path;

def unpack_Package(package, path):
    tar = tarfile.open(package, 'r:gz');
    for n in tar.getnames():
        tar.extract(n, os.path.abspath(path));
    tar.close();
    return os.path.abspath(path);

def install_Package(path):
    dest = os.path.join(FOLDER_PACKAGES, os.path.basename(path));
    shutil.copyfile(path, dest);
    return os.path.abspath(dest);

def delete_Package(name):
    os.remove(os.path.join(FOLDER_PACKAGES, name + '.tgz'));

def execute_Package(path, VNCOverride = False):
    hash = md5.new();
    hash.update(path + FOLDER_TEMP + str(time.localtime()));
    destpath = os.path.join(FOLDER_TEMP, string.upper(hash.hexdigest()));
    destpath = unpack_Package(path, destpath);
    t = get_CommandLine(os.path.join(destpath, SYM_QVM_FILE), VNCOverride);
    s = t[0]
    pid = os.spawnv(os.P_NOWAIT, s[0], s)
    f = open(os.path.join(destpath, SYM_PIDFILE) , 'w+');
    f.write(str(pid));
    f.flush()
    f.close();
    f = open(os.path.join(destpath, SYM_VNCID_FILE) , 'w+');
    f.write(str(t[1]));
    f.flush();
    f.close();
    f = open(os.path.join(destpath, SYM_VMM_FILE) , 'w+');
    f.write(str(t[2]));
    f.flush();
    f.close();
    os.waitpid(pid, 0);
    if int(t[1]) > 0:
        release_VNCID(int(t[1]));    
    for dirpath, dirs, files in os.walk(destpath):
        for l in files:
            os.remove(os.path.join(destpath, l));
    os.rmdir(destpath);
    return True;

def create_Machine(ID, package):
    return unpack_Package(os.path.join(FOLDER_PACKAGES, package + '.tgz'), os.path.join(FOLDER_MACHINES, ID));

def destroy_Machine(ID):
    if get_MachineState(ID) != SYM_STATE_INACTIVE:
        return;
    for dirpath, dirs, files in os.walk(os.path.join(FOLDER_MACHINES, ID)):
        for l in files:
            os.remove(os.path.join(os.path.join(FOLDER_MACHINES, ID), l));
    os.rmdir(os.path.join(FOLDER_MACHINES, ID));

def stop_Machine(ID):
    if not os.path.exists(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE)):
        return;
    f = open(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE), 'r');
    pid = int(string.replace(f.readline(), '\n', ''));
    f.close();
    f = open(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_VNCID_FILE), 'r');
    vncid = int(string.replace(f.readline(), '\n', ''));
    f.close();
    os.remove(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_VNCID_FILE))
    os.remove(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_VMM_FILE))
    os.remove(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE))
    try:
        if vncid > 0:
            release_VNCID(vncid);
        if os.name != 'nt':
            os.kill(pid, signal.SIGTERM);
        else:
            import win32api;
            import pywintypes;
            # 0x000001 = PROCESS_TERMINATE
            # 0x1F0FFF = PROCESS_ALL_ACCESS
            h = win32api.OpenProcess(1, 0, pid);
            win32api.TerminateProcess(h, 110);
    except:
        pass;
    return;

def commandLine_toString(lCommand):
    s = '';
    i = 1
    for i in lCommand:
        s = s + ' ' + i;
    return string.replace(s, ' ', '', 1);

def start_Machine(ID, VNCOverride = False):
    t = get_CommandLine(get_MachineFile(ID), VNCOverride);
    s = t[0] + ['-pidfile "' + os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE) + '"']
    #print 'N9.start_Machine() - DEBUG: ' + commandLine_toString(s);
    #print s
    if os.path.exists(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE)):
        return ['', 0, '', '']                    
    pid = os.spawnv(os.P_NOWAIT, s[0], s);
    while not os.path.exists(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE)):
        time.sleep(0.1);
    try:
        f = open(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE) , 'r');
        pid = int(f.read());
        f.close();
    finally:
        f = open(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_VNCID_FILE) , 'w+');
        f.write(str(t[1]));
        f.close();
        f = open(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_VMM_FILE) , 'w+');
        f.write(str(t[2]));
        f.close();
        #start_QMM(ID)
    return [commandLine_toString(s), pid, t[1], t[2]];

def get_RunningPackagesList():
    d = dict();
    for path, dirs, files in os.walk(FOLDER_TEMP):
        for folder in dirs:
            id = folder;
            path = os.path.join(FOLDER_TEMP, folder);
            f = open(os.path.join(path, SYM_PIDFILE), 'r');
            pid = string.replace(f.readline(), '\n', '')
            f.close();
            f = open(os.path.join(path, SYM_VNCID_FILE), 'r');
            vnc = string.replace(f.readline(), '\n', '')
            f.close();
            f = open(os.path.join(path, SYM_VMM_FILE), 'r');
            vmm = string.replace(f.readline(), '\n', '')
            f.close();
            data = ParseQVM(os.path.join(path, SYM_QVM_FILE), 'r');
            name = data[SYM_MACHINE_NAME]
            package = data[SYM_MACHINE_PACKAGE]
            refdata = [name, package, pid, vnc, vmm, path];
            d[id] = refdata
            
    return d;

def do_MonitorVM(ID):
    if not os.path.exists(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE)):
        return;
    f = open(os.path.join(os.path.join(FOLDER_MACHINES, ID), SYM_PIDFILE), 'r');
    pid = int(string.replace(f.readline(), '\n', ''));
    f.close();
    os.waitpid(pid);
    stop_Machine(ID);

def start_QMM(ID):
    if string.find(string.lower(sys.executable), 'python') == -1 :
        os.spawnv(os.P_NOWAIT, 'n9pmd.exe', ['n9pmd.exe', str(ID)]);
    else:
        os.spawnv(os.P_NOWAIT, sys.executable, [sys.executable, 'n9pmd.py', str(ID)])
    
configure(os.path.join(FOLDER_CONFIG, FILE_N9_CONFIG));
InitializeVNC(GLOBAL_VNC_MAP);

