#/usr/bin/python
#
# codename: N9 backup/restore utility
# (c) 2007 cyberlight systems research lab
import N9;
import time;
import os;
import os.path;
import tarfile;
import string;
import sys;

def print_Help():
    print """n9backup - codename:N9 backup management utility.
licensed under GPL v.2
(c) 2007 cyberlight systems
usage:
\tn9backup backup ID                -backups VM with given ID
\tn9backup backup -all              -backups all VMs
\tn9backup restore ID|-all DATE NUM -restores from backup with given VM ID and order number NUM built at date DATE
\tn9backup list ID|-all|-full       -displays backup tree for given VM (ID), all VMs (-all) or full tree (-full)
where:
\tID    - ID of VM
\tDATE  - backup date in YYYY-MM-DD format
\tNUM   - at date DATE backup number NUM (1,2, 10, 967...). Not enter "00000967", simply "967"
\t-all  - alias for backuping/restoring displaying all actual VMs
\t-full - alias for displaying full backups tree instead context root of ID"""

def print_Error():
    print 'Invalid arguments. Run n9mkpkg with "-help" switch for exploring usage';


def tfill(s, width=0):
    if width == 0:
        return s;
    for i in range(1, width+1):
        s = ' ' + s;
    return s;

def get_DateStr():
    s = time.strftime('%Y-%m-%d' , time.localtime());
    return s

def get_BackupRoot():
    return N9.FOLDER_BACKUPS_ROOT;

def get_BackupName(ID):
    i = 1
    d =  get_DateStr()
    topfolder = os.path.join(get_BackupRoot(), ID);
    if not os.path.exists(topfolder):
        os.mkdir(topfolder);
    currfolder = os.path.join(topfolder, d);
    if not os.path.exists(currfolder):
        os.mkdir(currfolder);
    else:
        for top, subfolders, files in os.walk(currfolder):
            for file in files:
                i = i + 1;
    num = string.zfill(str(i), 8);
    return os.path.join(currfolder, d + '-' + ID + '-' + num + '.tgz');
                

def get_BackupsFor(ID):
    return os.path.join(get_BackupRoot(), ID);

def backup(root, ID):
    fname = get_BackupName(ID);
    tar = tarfile.open(fname, 'w:gz');
    tar.add(root, os.path.basename(root));
    tar.close();
    return os.path.basename(fname);
    
def restore(ID, date, num):
    root = get_BackupsFor(ID);
    backups = os.path.join(root, date);
    znum = string.zfill(num, 8);
    bname = date + '-' + ID + '-' + znum + '.tgz'
    fname = os.path.join(backups, bname);
    if not os.path.exists(fname):
        return 'Backup "' + bname + '" not exists for target ' + ID + ' at date ' + date;
    if string.upper(ID) != 'ALL':
        dest = N9.FOLDER_MACHINES;
    else:
        dest = os.path.abspath(os.path.normpath(os.path.join(N9.FOLDER_MACHINES, '..')));
    tar = tarfile.open(fname, 'r:gz');
    names = tar.getnames()
    for x in names:
        tar.extract(x, dest);
    tar.close();
    return 'Restoring ' + ID + ' from ' + bname + ' succeeded\n' + 'Date: ' + date + '; order number: ' + znum +'\nbackup: ' + fname;


def folderlist(root, depth):
    s = ''
    if not os.path.exists(root):
        return s;
    s = s + tfill(os.path.basename(root), depth) + '\n';
    names = os.listdir(root);
    for f in names:
        if os.path.isdir(os.path.join(root, f)):
            s = s + folderlist(os.path.join(root, f), depth + 1);
        else:
            if os.path.isfile(os.path.join(root, f)):
                s = s + tfill(f, depth + 1) + '\n';
    return s;

if __name__ == '__main__':
    if len(sys.argv) <= 1:
        print_Help();
    else:
        if len(sys.argv) == 2 and (string.upper(sys.argv[1]) == '--HELP' or
         string.upper(sys.argv[1]) == '-HELP' or
         string.upper(sys.argv[1]) == '-H' or
         string.upper(sys.argv[1]) == '-?' or
         string.upper(sys.argv[1]) == '/?'):
            print_Help();
        else:
            if len(sys.argv) == 3 and string.upper(sys.argv[1]) == 'BACKUP':
                if string.upper(sys.argv[2]) == '-ALL':
                    src = N9.FOLDER_MACHINES
                    id  = 'all';
                else:
                    src = os.path.join(N9.FOLDER_MACHINES, sys.argv[2]);
                    id  = sys.argv[2];
                print 'backing up ' + id
                fname = backup(src, id);
                print 'VM "' + id + '" successfully backed up to "' + fname + '"';
                sys.exit()

            if len(sys.argv) == 3 and string.upper(sys.argv[1]) == 'LIST':
                if string.upper(sys.argv[2]) == '-ALL':
                    d = folderlist(get_BackupsFor('all'), 0);
                else:
                    if string.upper(sys.argv[2]) == '-FULL':
                        d = folderlist(get_BackupRoot(), 0);
                    else:
                        d = folderlist(get_BackupsFor(sys.argv[2]), 0);
                print d;
                sys.exit()
                    
            if len(sys.argv) == 5 and string.upper(sys.argv[1]) == 'RESTORE':
                print 'restoring ... '
                print restore(sys.argv[2], sys.argv[3], sys.argv[4]);
                sys.exit()
            print_Error()


