# N9 command-line interface utilities
# N9 Package manager
# 2007 cyberlight systems research lab

import sys;
import string
import N9;
import os.path;
import shutil;


def print_Help():
    print """n9vmctl - codename:N9 virtual machine management utility.
licensed under GPL v.2
(c) 2007 cyberlight systems
usage:
\tn9vmctl create ID PACKAGE-NAME - install new or reinstall existed VM
\tn9vmctl destroy ID             - destroy VM
\tn9vmctl start ID [-vnc]        - powers on VM
\tn9vmctl stop ID                - hardly power off VM and cleanup flag files
\tn9vmctl state ID               - query VM state
\tn9vmctl list                   - display installed VM info
\tn9vmctl list --temp            - display running packages info
\tn9vmctl export ID PACKAGE-FILE - export installed VM to package
\tn9vmctl set ID param=value     - change configuration of VM
\tn9vmctl display ID             - display VM config

where:
\tPACKAGE-FILE    - .tgz package file name
\tPACKAGE-NAME    - installed package name
\tID              - machine ID
\tparam=value     - VM configuration file parameter-value pair
\t[-vnc]          - optional for enforcing run in headless VNC server mode"""

def print_Error():
    print 'Invalid arguments. Run n9vmctl with "-help" switch for exploring usage';

 
if __name__ == '__main__':
    VNC = False;
    if len(sys.argv) <= 1:
        print_Help();
    else:
        if len(sys.argv) == 2 and (string.upper(sys.argv[1]) == '--HELP' or
         string.upper(sys.argv[1]) == '-HELP' or
         string.upper(sys.argv[1]) == '-H' or
         string.upper(sys.argv[1]) == '-?' or
         string.upper(sys.argv[1]) == '/?'):
            print_Help();
        else:
            if len(sys.argv) > 2:
                if string.upper(sys.argv[1]) == 'CREATE':
                    N9.create_Machine(sys.argv[2], sys.argv[3]);
                    
                if string.upper(sys.argv[1]) == 'DESTROY':
                    N9.destroy_Machine(sys.argv[2]);
                    
                if string.upper(sys.argv[1]) == 'START':
                    if len(sys.argv) >= 4:
                        if string.upper(sys.argv[3])== '-VNC':
                            VNC = True
                    N9.start_Machine(sys.argv[2], VNC);
                    
                if string.upper(sys.argv[1]) == 'STOP':
                    N9.stop_Machine(sys.argv[2]);
                    
                if string.upper(sys.argv[1]) == 'STATE':
                    l = N9.get_MachineState(sys.argv[2]);
                    print sys.argv[2] + ' ' + l;
                    
                if string.upper(sys.argv[1]) == 'LIST' and len(sys.argv) > 2:                    
                    if string.upper(sys.argv[2]) == '--TEMP':
                        d = N9.get_RunningPackagesList();
                        print 'ID\tNAME\tPACKAGE\tPID\tVNC\tMONITOR'
                        for k in d.keys():
                            l = d[k];
                            print k + '\t' + l[0] + '\t' + l[1] + '\t' + l[2] + '\t' + l[3] + '\t' + l[4]
                            
                if string.upper(sys.argv[1]) == 'EXPORT':
                    params = N9.ParseQVM(N9.get_MachineFile(sys.argv[2]))
                    srcpath = os.path.dirname(N9.get_MachineFile(sys.argv[2]))
                    src = os.path.join(srcpath, os.path.basename(sys.argv[3]))
                    N9.create_Package(params, src);
                    shutil.copyfile(src, sys.argv[3]);
                    N9.unpack_Package(src, srcpath);
                    os.remove(src);
                
                if string.upper(sys.argv[1]) == 'SET':
                    l = sys.argv[3];
                    l = string.replace(l, '\t', '');
                    l = string.replace(l, '"', '');
                    t = string.split(l, '=');
                    key = t[0];
                    value = t[1]
                    params = N9.ParseQVM(N9.get_MachineFile(sys.argv[2]))
                    if key in N9.QVM_SYMBOLS:
                        params[key] = value;
                        f = open(N9.get_MachineFile(sys.argv[2]), 'w+');
                        for i in params.keys():
                            if len(params[i]) > 0:
                                s = i + '=' + params[i] + '\n';
                                f.write(s);
                        f.flush()
                        f.close();

                if string.upper(sys.argv[1]) == 'DISPLAY':
                    d = N9.ParseQVM(N9.get_MachineFile(sys.argv[2]));
                    print 'Virtual Machine ' + sys.argv[2] + ' configuration:'
                    for k in d.keys():
                        print '\t' + k + '\t' + d[k];
            else:
                if string.upper(sys.argv[1]) == 'LIST' and len(sys.argv) == 2:
                    d = N9.get_MachinesList();
                    print 'ID\tPACKAGE\t\tNAME\t\tSTATE\tRAM\tVNC'
                    for k in d.keys():
                        l = d[k];
                        vnc = '0'
                        if l.has_key(N9.SYM_MACHINE_VNCID):
                            vnc = l[N9.SYM_MACHINE_VNCID];
                        
                        print k + '\t' + l[N9.SYM_MACHINE_PACKAGE] + '\t\t' + l[N9.SYM_MACHINE_NAME] + '\t\t' + l[N9.SYM_MACHINE_STATE] + '\t' + l[N9.SYM_MACHINE_RAM] + '\t' + vnc
                else:
                    print_Error();
            
